'use strict';

// var moment = require('moment-timezone');
// console.log(moment(new Date()).tz("Australia/Sydney").format());

//configurations
const path = require('path');
const express = require('express');
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const logger = require('./utils/logger');
const jsonResponse = require('./utils/json-response');
const config = require('./config');
const errors = require('./utils/dz-errors');
const cors = require('cors');
const cron = require('node-cron');
const request = require('request');


//routes
const routes = require('./routes/index');
const authBackend = require('./routes/backend/auth');
const settingBackend = require('./routes/backend/settings');
const language = require('./routes/backend/language');
const label = require('./routes/backend/label');
const cms = require('./routes/backend/cms');
const helpCategoryBackend = require('./routes/backend/help-category');
const helpBackend = require('./routes/backend/help');
const email = require('./routes/backend/email');
const push = require('./routes/backend/push');
const sms = require('./routes/backend/sms');
const role = require('./routes/backend/role');
const user = require('./routes/backend/user');
const screen = require('./routes/backend/screen');
const currency = require('./routes/backend/currency');
const measurement = require('./routes/backend/measurement');
const tax = require('./routes/backend/tax');
const vehicleType = require('./routes/backend/vehicle-type');
const vehicleColor = require('./routes/backend/vehicle-color');
const parkingSpotType = require('./routes/backend/parking-spot-type');
const parkingSpotCoverage = require('./routes/backend/parking-spot-coverage');
const parkingSpotTerrain = require('./routes/backend/parking-spot-terrain');
const parkingSpotAmenity = require('./routes/backend/parking-spot-amenity');
const accessInstruction = require('./routes/backend/access-instruction');
const pricing = require('./routes/backend/pricing');
const bank = require('./routes/backend/bank');
const accountType = require('./routes/backend/bank-account-type');
const paymentFrequency = require('./routes/backend/payment-frequency');
const customer = require('./routes/backend/customer');
const moduleBack = require('./routes/backend/module');
const accessRight = require('./routes/backend/access-right');
const loginlog = require('./routes/backend/loginlog');
const booking = require('./routes/backend/booking');
const propertyBack = require('./routes/backend/property');
const dashboard = require('./routes/backend/dashboard');
const report = require('./routes/backend/report');
const backup = require('./routes/backend/backup');
const payouts = require('./routes/backend/payouts');

// FOR API
const authAPI = require('./routes/api/auth');
const card = require('./routes/api/card');
const helpCategory = require('./routes/api/help-category');
const help = require('./routes/api/help');
const languageAPI = require('./routes/api/language');
const labelAPI = require('./routes/api/label');
const apiCms = require('./routes/api/cms');
const measurementAPI = require('./routes/api/measurement');
const vehicleTypeAPI = require('./routes/api/vehicle-type');
const vehicleColorAPI = require('./routes/api/vehicle-color');
const vehicle = require('./routes/api/vehicle');
const property = require('./routes/api/property');
const parking = require('./routes/api/parking');
const accessInstructionAPI = require('./routes/api/access-instruction');
const seeker = require('./routes/api/seeker');
const owner = require('./routes/api/owner');
const pricingAPI = require('./routes/api/pricing');
const socket = require('./routes/api/socket');
const notificationAPI = require('./routes/api/notification');
const cronAPI = require('./routes/api/cron');

//other configurations
const passport = require('passport');
const favicon = require('serve-favicon');
const multiparty = require('connect-multiparty');
const upload = require('express-fileupload');
const multipartyMiddleWare = multiparty();

//express configurations
const app = express();
app.use(favicon(path.join(__dirname, './public/img', 'favicon.ico')));
app.use(express.static(path.join(__dirname, 'public')));
app.use(bodyParser.json({ limit: '50mb' }));
app.use(bodyParser.urlencoded({ extended: true, limit: '50mb' }));
app.use(cookieParser());
app.use(require('express-session')({
	secret: 'keyboard cat',
	resave: false,
	saveUninitialized: false
}));
app.use(cors());
app.use(passport.initialize());
app.use(passport.session());
app.use(multipartyMiddleWare);


app.use((req, res, next) => {
	console.log(`** requested :  '${new Date().toISOString()}' '${req.originalUrl}' **\n`)
	next()
});

// import routes
app.use('/', routes);
app.use('/backend/auth', authBackend);
app.use('/backend/settings', settingBackend);
app.use('/backend/language', language);
app.use('/backend/label', label);
app.use('/backend/cms', cms);
app.use('/backend/help-category', helpCategoryBackend);
app.use('/backend/help', helpBackend);
app.use('/backend/email', email);
app.use('/backend/push', push);
app.use('/backend/sms', sms);
app.use('/backend/role', role);
app.use('/backend/user', user);
app.use('/backend/screen', screen);
app.use('/backend/currency', currency);
app.use('/backend/measurement', measurement);
app.use('/backend/tax', tax);
app.use('/backend/vehicle-type', vehicleType);
app.use('/backend/vehicle-color', vehicleColor);
app.use('/backend/parking-spot-type', parkingSpotType);
app.use('/backend/parking-spot-coverage', parkingSpotCoverage);
app.use('/backend/parking-spot-terrain', parkingSpotTerrain);
app.use('/backend/parking-spot-amenity', parkingSpotAmenity);
app.use('/backend/access-instruction', accessInstruction);
app.use('/backend/pricing', pricing);
app.use('/backend/bank', bank);
app.use('/backend/bank-account-type', accountType);
app.use('/backend/payment-frequency', paymentFrequency);
app.use('/backend/customer', customer);
app.use('/backend/module', moduleBack);
app.use('/backend/access-right', accessRight);
app.use('/backend/loginlog', loginlog);
app.use('/backend/booking', booking);
app.use('/backend/property', propertyBack);
app.use('/backend/dashboard', dashboard);
app.use('/backend/report', report);
app.use('/backend/backup', backup);
app.use('/backend/payouts', payouts);

// FOR API
app.use('/api/auth', authAPI);
app.use('/api/card', card);
app.use('/api/language', languageAPI);
app.use('/api/label', labelAPI);
app.use('/api/cms', apiCms);
app.use('/api/help-category', helpCategory);
app.use('/api/help', help);
app.use('/api/measurement', measurementAPI);
app.use('/api/vehicle-type', vehicleTypeAPI);
app.use('/api/vehicle-color', vehicleColorAPI);
app.use('/api/vehicle', vehicle);
app.use('/api/property', property);
app.use('/api/parking', parking);
app.use('/api/access-instruction', accessInstructionAPI);
app.use('/api/seeker', seeker);
app.use('/api/pricing', pricingAPI);
app.use('/api/owner', owner);
app.use('/api/socket', socket);
app.use('/api/notification', notificationAPI);
app.use('/api/cron', cronAPI);


app.use(upload());

var swaggerUi = require("swagger-ui-express"),
	swaggerDocument = require("./swagger.json");

app.use("/api-swagger", swaggerUi.serve, swaggerUi.setup(swaggerDocument));

// catch 404 and forward to error handler
app.use((req, res) => {
	logger('Error: No route found or Wrong method name');
	jsonResponse(res, errors.resourceNotFound(true), null);
});

// cron.schedule('* * * * *', function() {
//     request.post({
// 	    headers: {
// 	        'content-type': 'application/x-www-form-urlencoded',
// 	    },
// 	    url: config.base_url+'api/seeker/complete-reservation',
//     }, function(error, response, body) {
//     });
// });

cron.schedule('* * * * *', function () {
	request.post({
		headers: {
			'content-type': 'application/x-www-form-urlencoded',
		},
		url: config.base_url + 'api/owner/cron-for-overage-fees',
	}, function (error, response, body) {
	});
});

cron.schedule('* * * * *', function () {
	request.post({
		headers: {
			'content-type': 'application/x-www-form-urlencoded',
		},
		url: config.base_url + 'api/seeker/cron-for-before-after-push',
	}, function (error, response, body) {
	});
});

cron.schedule('* * * * *', function () {
	request.post({
		headers: {
			'content-type': 'application/x-www-form-urlencoded',
		},
		url: config.base_url + 'api/seeker/cancel-reservation-from-cron',
	}, function (error, response, body) {
	});
});

// FOR EVERY MONTH PAYOUT
cron.schedule('0 0 1 * *', function () {
	request.get({
		headers: {
			'content-type': 'application/x-www-form-urlencoded',
		},
		url: config.base_url + 'api/cron/payout?type=monthly',
	}, function (error, response, body) {
	});
});

// FOR EVERY QUARTER PAYOUT
cron.schedule('0 0 1 */3 *', function () {
	request.get({
		headers: {
			'content-type': 'application/x-www-form-urlencoded',
		},
		url: config.base_url + 'api/cron/payout?type=quarterly',
	}, function (error, response, body) {
	});
});

// FOR EVERY ANNUALLY PAYOUT
cron.schedule('0 0 1 1 *', function () {
	request.get({
		headers: {
			'content-type': 'application/x-www-form-urlencoded',
		},
		url: config.base_url + 'api/cron/payout?type=annually',
	}, function (error, response, body) {
	});
});

// development error handler
// will print stacktrace

if (app.get('env') === 'development') {
	app.use((err, req, res) => {
		res.status(err.status || 500);
		res.render('error', {
			message: err.message,
			error: err
		});
	});
}

// production error handler
// no stack traces leaked to user
app.use((err, req, res) => {
	res.status(err.status || 500);
	res.render('error', {
		message: err.message,
		error: {}
	});
});

module.exports = app;




