const dbConstants = require('../constants/db-constants');
const { reservationCharges } = require('../utils/charges');
const query = require('../utils/query-creator-api');
const config = require('../config');
const distance = require('google-distance');
distance.apiKey = config.google_key;
/**
 * create reservation charge with stripe gateways
 * @param {Object} param0 
 * @returns stripeRes, gateway_charge, company_earnings, owner_earnings, error: null
 */
exports.createReservationChargesStripe = async ({ settings, is_payment_confirmed, amount, customer_stripe_id, card_id, owner_connected_account, order_id }) => {
    let response = {
        stripeRes: null,
        gateway_charge: 0, company_earnings: 0, owner_earnings: 0,
        error: null
    }
    try {
        if (!settings) {
            settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
        }
        let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);

        let { gateway_charge, company_earning, owner_earning } = reservationCharges(amount, settings.company_percentage)
        console.log(amount, gateway_charge, company_earning, owner_earning)
        let chargeAmount = parseInt(Math.round(parseFloat(amount) * 100).toFixed(2));
        let stripe_owner_earning = parseInt(Math.round(parseFloat(owner_earning) * 100).toFixed(2));
        let stripeRes = await stripe.paymentIntents.create({
            amount: chargeAmount,
            currency: 'usd',
            customer: customer_stripe_id,
            payment_method: card_id,
            confirmation_method: "manual",
            transfer_data: {
                amount: stripe_owner_earning,
                destination: owner_connected_account,
            },
        });

        if (is_payment_confirmed == true) {
            const paymentIntentConfirm = await stripe.paymentIntents.confirm(
                stripeRes.id,
                { payment_method: "pm_card_visa" }

            );
            stripeRes = paymentIntentConfirm;
        }

        return {
            ...response,
            stripeRes: stripeRes,
            gateway_charge, company_earning, owner_earning,
            error: null
        }
    } catch (error) {
        return { ...response, error: error }
    }
}

/**
 * Payment confirmed or captured
 * @param {String} transaction_id 
 * @returns stripe response
 */
exports.stripePaymentConfirmed = async (settings, transaction_id) => {
    let response = { error: null };
    try {
        if (!settings) {
            settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
        }
        let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);
        let res = await stripe.paymentIntents.confirm(
            transaction_id,
            { payment_method: "pm_card_visa" }

        );
        return { ...response, ...res }
    } catch (error) {
        return { ...response, error };
    }
}