'use strict';

const config = require('./../../config');
const logger = require('./../../utils/logger');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator-api');
const _ = require('underscore');
const AWS = require('aws-sdk');
AWS.config.update({
    accessKeyId: config.aws.keyId,
    secretAccessKey: config.aws.key,
    region: config.aws.sesRegion
});
const ses = new AWS.SES({apiVersion: '2010-12-01'});
const labels = require('./../../utils/labels.json');

const setupEmail = async(requestParam) => {
    const params = {
        Destination: {
            ToAddresses: requestParam.to_email
        },
        Message: {
            Body: {
                Html: {
                    Charset: 'UTF-8',
                    Data: requestParam.description
                }
            },
            Subject: {
                Charset: 'UTF-8',
                Data: requestParam.subject
            }
        },
        ReturnPath: requestParam.from_email,
        Source: requestParam.from_email,
    };

    ses.sendEmail(params, (err, data) => {
        if (err) {
            return console.log(err, err.stack);
        } else {
            console.log("Email sent.", data);
        }
    });
};

/*
	Name : Send Email
	Purpose : Send Email
	Original Author : Gaurav Patel
	Created At : 3rd Sept 2019
*/
const sendEmail = async(requestParams, done) => {
    return new Promise(async(resolve, reject) => {
        try {
            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, {} );
            let template = await query.selectWithAndOne(dbConstants.dbSchema.email_templates, {code: requestParams.template_code}, {
                from_name: 1,
                from_email: 1,
                email_subject: 1,
                description: 1,
            });
            if (template) {
                let emailSubject = template.email_subject[requestParams.code];

                let emailTemplate = template['description'][requestParams.code];

                emailTemplate = emailTemplate.replace('#FACEBOOK#', settings.fb_url);
                emailTemplate = emailTemplate.replace('#TWITTER#', settings.twitter_url);
                emailTemplate = emailTemplate.replace('#INSTAGRAM#', settings.instagram_url);
                emailTemplate = emailTemplate.replace('#LINKEDIN#', settings.linkedin_url);
                emailTemplate = emailTemplate.replace('#YOUTUBE#', settings.youtube_url);
                emailTemplate = emailTemplate.replace(/#SUPPORT_EMAIL#/g, settings.support_email);
                emailTemplate = emailTemplate.replace('#ADDRESS#', settings.company_address);
                emailTemplate = emailTemplate.replace(/#CALL_US#/g, settings.call_us);

                if (requestParams.template_code == 'CUS_SIGN_UP' || requestParams.template_code == 'DRI_SIGN_UP' || requestParams.template_code == 'DRI_ACTIVATION') {
                    emailTemplate = emailTemplate.replace('#NAME#', requestParams.name);
                }
                if (requestParams.template_code == 'CONTACT_US_REPLY') {
                    emailTemplate = emailTemplate.replace('#NAME#', requestParams.name);
                    emailTemplate = emailTemplate.replace('#QUESTION#', requestParams.question);
                    emailTemplate = emailTemplate.replace('#ANSWER#', requestParams.answer);
                }
                if (requestParams.template_code == 'ORDER_STATUS' || requestParams.template_code == 'EMAIL_RECEIPT') {
                    emailSubject = emailSubject.replace('#ORDER_ID#', requestParams.order_id);
                    emailSubject = emailSubject.replace('#ORDER_STATUS#', requestParams.status);

                    emailTemplate = emailTemplate.replace('#NAME#', requestParams.name);
                    emailTemplate = emailTemplate.replace(/#ORDER_STATUS#/g, requestParams.status);
                    emailTemplate = emailTemplate.replace(/#ORDER_ID#/g, requestParams.order_id);
                    emailTemplate = emailTemplate.replace('#PAYMENT_TYPE#', requestParams.payment_type);
                    emailTemplate = emailTemplate.replace('#SERVICE_TYPE#', requestParams.service_type);
                    emailTemplate = emailTemplate.replace(/#CURRENCY#/g, settings.default_currency);
                    emailTemplate = emailTemplate.replace(/#TOTAL#/g, requestParams.total);
                    emailTemplate = emailTemplate.replace('#PICKUP_DELIVERY#', requestParams.pickup_delivery_details);
                    emailTemplate = emailTemplate.replace('#ADDONS#', requestParams.addons);
                    emailTemplate = emailTemplate.replace('#CHARGES#', requestParams.charges);
                    emailTemplate = emailTemplate.replace(/#SUB_TOTAL#/g, requestParams.sub_total);
                }

                setupEmail({
                    to_email: [requestParams.email],
                    from_email: template.from_name + ' <' + template.from_email + '>',
                    subject: emailSubject,
                    description: emailTemplate
                });

                return false;
            } else {
                return false;
            }
        } catch (error) {
            console.log(error);
            return false;
        }
    });
};

/*
    Name : Send Email Order Status
    Purpose : Send Email Order Status
    Original Author : Gaurav Patel
    Created At : 17th Feb 2021
*/
const sendEmailOrderStatus = async(requestParam, code) => {
    return new Promise(async(resolve, reject) => {
        try {
            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, {} );

            // Customer
            let customer = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1, name:1, email:1}, { created_at: 1 });
            customer = customer==null ? {} : customer;

            // Service
            let service = await query.selectWithAndOne(dbConstants.dbSchema.services, { service_id: requestParam.service_id }, { _id: 0, service_id: 1, title:1}, { created_at: 1 });
            service = service==null ? {} : service

            // Driver
            let driver = {};
            if(requestParam.status != 'new' && requestParam.status != 'cancelled'){
                driver = await query.selectWithAndOne(dbConstants.dbSchema.drivers, { driver_id: requestParam.driver_id }, { _id: 0, driver_id: 1, name:1, email:1, mobile_country_code:1, mobile:1}, { created_at: 1 });
                driver = driver==null ? {} : driver
            }

            // Pickup/Delivery/Return delivery Info;
            _.each(requestParam.pickup_details, (elem, index) => {
                elem.type = 'Pickup'
            });
            _.each(requestParam.delivery_details, (elem1, index) => {
                elem1.type = 'Delivery'
            });
            _.each(requestParam.return_delivery_info, (elem2, index) => {
                elem2.type = 'Return Delivery'
            });
            let arr = requestParam.pickup_details;
            arr.push(requestParam.delivery_details);
            arr.push(requestParam.return_delivery_info);
            arr = _.flatten(arr)

            let pickup_delivery_details = '<table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="padding: 25px;border-collapse:collapse;"><tbody>';

            _.each(arr, (elem, index) => {
                let count = index;
                if(elem.type == 'Delivery' || elem.type == 'Return Delivery'){
                    count = 0;
                }
                let cnt = index/2;
                if(cnt == 0){
                    pickup_delivery_details += '<tr>'
                }
                let num = count==0 ? '' : count
                pickup_delivery_details += '<td style="padding: 15px 10px;font-size: 18px;color: #000;"><b>'+elem.type+' '+num+'</b> <p style="font-size: 16px;margin: 5px 0;">'+elem.name+' </p><p style="font-size: 16px;margin: 5px 0;">Phone : '+elem.mobile_country_code+elem.mobile+'</p><p style="font-size: 16px;margin: 5px 0;">Address : '+elem.location+'</p></td>'
                if(cnt != 0){
                    pickup_delivery_details += '</tr>'
                }
            });
            pickup_delivery_details += '</tbody></table>'

            // Assigned Driver
            if(requestParam.status != 'new' && requestParam.status != 'cancelled'){
                pickup_delivery_details += '<tr><td style="padding: 15px 10px;font-size: 18px;color: #000;"><b>Assigned Driver</b> <p style="font-size: 16px;margin: 5px 0;">'+driver.name+' </p><p style="font-size: 16px;margin: 5px 0;">Call : '+driver.mobile_country_code+driver.mobile+'</p></td></tr>'
            }
            pickup_delivery_details += '</tbody></table>'

             // Addons
            let addons = '';
            if(requestParam.addons.length > 0){
                let subAddon = ''
                _.each(requestParam.addons, (elem, index) => {
                    let price = elem.price;
                    let charge_info = requestParam.charge_info ? requestParam.charge_info : {};
                    if(charge_info[elem.addon_id]){
                        price = charge_info[elem.addon_id]
                    }
                    subAddon += '<tr><td style="width: 10%;padding: 15px 10px;border: 1px solid #a4b7c1;">'+(index+1)+'</td><td style="width: 50%;padding: 15px 10px;border: 1px solid #a4b7c1;">'+elem.title+' </td><td style="width: 20%;padding: 15px 10px;border: 1px solid #a4b7c1;">'+elem.count+'</td><td style="width: 20%;padding: 15px 10px;border: 1px solid #a4b7c1;">'+price+'</td></tr>'
                });
                addons = '<table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="padding: 25px;border-collapse:collapse;"><tbody><tr><td style="padding: 15px 10px;font-size: 22px;color: #000;"><b>Addons Details </b></td></tr></tbody></table><section style="width: 100%;float: right;"><table class="break-page" style="page-break-after: always;width: 96%;border-collapse: collapse;font-weight: 900;font-size: 14px;padding: 15px;margin: 0 auto;" border=""><tbody><tr><td style="width: 10%;padding: 15px 10px;border: 1px solid #a4b7c1;">S.No</td><td style="width: 50%;padding: 15px 10px;border: 1px solid #a4b7c1;">Title</td><td style="width: 20%;padding: 15px 10px;border: 1px solid #a4b7c1;">Count </td><td style="width: 20%;padding: 15px 10px;border: 1px solid #a4b7c1;">Price </td></tr>'+subAddon+'</tbody></table></section>'
            }

            //charges
            let extra_charges = '';
            _.each(requestParam.tax_info, (elem, index) => {
                extra_charges += '<tr><td style="padding:5px 10px;font-weight: 900;">'+elem.title+'['+elem.percentage+'%]</td><td style="padding:5px 10px;text-align: right;">'+settings.default_currency+elem.value+'</td></tr>'
            });
            if(requestParam.discount > 0){
                extra_charges += '<tr><td style="padding:5px 10px;font-weight: 900;">Discount</td><td style="padding:5px 10px;text-align: right;">'+settings.default_currency+requestParam.discount+'</td></tr>'
            }
            let keys = Object.keys(requestParam.charge_info);
            _.each(keys, (elem) => {
                let title = '';
                if(elem == 'wrong_address_charge'){
                    title = labels.LBL_WRONG_ADDRESS_CHARGE['MESSAGE'][code]
                }
                else if(elem == 'restrict_zone_charge'){
                    title = labels.LBL_RESTRICT_ZONE_CHARGE['MESSAGE'][code]
                }
                else if(elem == 'overtime_charge'){
                    title = labels.LBL_OVERTIME_CHARGE['MESSAGE'][code]
                }
                else if(elem == 'parking_charge'){
                    title = labels.LBL_PARKING_CHARGE['MESSAGE'][code]
                }
                else if(elem == 'waiting_charge'){
                    title = labels.LBL_WAITING_CHARGE['MESSAGE'][code]
                }
                else if(elem == 'oversized_item_charge'){
                    title = labels.LBL_OVERSIZE_ITEM_CHARGE['MESSAGE'][code]
                }
                if(title != ''){
                    extra_charges += '<tr><td style="padding:5px 10px;font-weight: 900;">'+title+'</td><td style="padding:5px 10px;text-align: right;">'+settings.default_currency+requestParam.charge_info[elem]+'</td></tr>'
                }
            });
            let charges = '<section style="page-break-after: always; width: 96%; border-collapse: collapse; font-weight: 900; font-size: 14px; padding: 15px; margin: 0 auto;"><table style="width: 60%; float: right; border-collapse: collapse; font-size: 14px; box-shadow: 1px 0 5px #c3c3c3;"><tbody><tr> <td style="padding: 5px 10px; font-weight: 900;">Delivery charge</td><td style="padding: 5px 10px; text-align: right;">'+settings.default_currency+(requestParam.sub_total).toFixed(2)+'</td></tr>'+extra_charges+'<tr><td style="padding: 5px 10px; font-weight: 900; font-size: 20px; background-color: #f0f3f5; border-top: 3px solid #1369b5;">Amount Payable</td><td style="padding: 5px 10px; font-weight: 900; text-align: right; font-size: 20px; background-color: #f0f3f5; border-top: 3px solid #1369b5;">'+settings.default_currency+(requestParam.total).toFixed(2)+'</td></tr></tbody></table></section>'

            let status = 'Placed';
            if(requestParam.status == 'accepted'){
                status = 'Accepted';
            }
            if(requestParam.status == 'pickedup'){
                status = 'Picked Up';
            }
            if(requestParam.status == 'completed'){
                status = 'Completed';
            }
            let obj = {
                template_code: requestParam.template_code ? requestParam.template_code : 'ORDER_STATUS',
                code: code,
                name:customer.name,
                email:customer.email,
                order_id:requestParam.order_id,
                payment_type:requestParam.payment_type,
                service_type:service.title[code],
                total:(requestParam.total).toFixed(2),
                sub_total:(requestParam.sub_total).toFixed(2),
                discount:requestParam.discount,
                pickup_delivery_details:pickup_delivery_details,
                addons:addons,
                status:status,
                charges:charges,
            }
            if(requestParam.template_code == 'DOWNLOAD_PDF'){
                resolve(obj);
                return;
            }
            else{
                sendEmail(obj);
            }
        } catch (error) {
            console.log(error);
            return false;
        }
    });
};


module.exports = {
    sendEmail,
    setupEmail,
    sendEmailOrderStatus
};