'use strict';

const config = require('./../../config');
const logger = require('./../../utils/logger');
const errors = require('./../../utils/dz-errors-api');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator-api');
const moment = require('moment');
const timeZone = require('moment-timezone');
let asyncLoop = require('async');
let _ = require('underscore');
const Notification_log = require('./../../models/notification-log');
const FCM = require('fcm-node');

/*
	Name : Assign Notification
	Purpose : Assign Notification
	Original Author : Gaurav Patel
	Created At : 24th May 2021
*/
const sendNotification = async(requestParam) => {
    console.log("sendNotification")
    return new Promise(async(resolve, reject) => {
        try {
            const fcm = new FCM(config.push_server_key);
            const message = {
                registration_ids: requestParam.device_token,
                collapse_key: 'green',
                data: {
                    title: requestParam.title,
                    body: requestParam.message,
                    type: requestParam.type,
                    user_id: requestParam.user_id,
                    sender_id: requestParam.sender_id,
                    reservation_id: requestParam.reservation_id,
                    user_type: requestParam.user_type,
                    reservation_status: requestParam.reservation_status,
                },
                notification: {
                    title: requestParam.title,
                    body: requestParam.message,
                    type: requestParam.type,
                    user_id: requestParam.user_id,
                    sender_id: requestParam.sender_id,
                    reservation_id: requestParam.reservation_id,
                    user_type: requestParam.user_type,
                    reservation_status: requestParam.reservation_status,
                }
            };
            fcm.send(message, function(error, response) {
                console.log(error);
                console.log(response);
            }) 
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

/*
    Name : Assign Notification
    Purpose : Assign Notification
    Original Author : Gaurav Patel
    Created At : 20th Aug 2020
*/
const setLogs = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            asyncLoop.forEachSeries(requestParam.user_id, async function(rec, Callback_s1) {
                let obj = {
                    user_type: requestParam.user_type,
                    user_id: rec,
                    message: requestParam.message,
                    description: requestParam.description,
                    type: requestParam.type,
                    address: requestParam.address,
                    sender_id: requestParam.sender_id,
                    reservation_id: requestParam.reservation_id,
                    reservation_status: requestParam.reservation_status,
                };
                if(requestParam.profile_picture){
                    obj.profile_picture = requestParam.profile_picture
                }
                if(requestParam.name){
                    obj.name = requestParam.name
                }
                await query.insertSingle(dbConstants.dbSchema.notification_logs, obj);
                Callback_s1();
            },function(){
                return false;
            });
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

/*
    Name : List
    Purpose : List
    Original Author : Gaurav Patel
    Created At : 29th Sep 2020
*/
const list = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            let columnValue = {
                user_id: requestParam.user_id
            }
            if(requestParam.type == 'unread'){
                columnValue.read_status = 'unread'
            }
            let noti = await query.selectWithAnd(dbConstants.dbSchema.notification_logs, columnValue, { _id: 0, notification_log_id:1, user_id: 1, message: 1, description:1, created_at: 1, read_status:1, type:1, address:1, profile_picture:1, name:1, user_type:1, sender_id:1, reservation_id:1, reservation_status:1}, { created_at: -1 });
            noti = JSON.parse(JSON.stringify(noti));
            let today = timeZone(new Date()).tz(config.time_zone).format('YYYY-MM-DD');
            let todayDate = timeZone(new Date()).tz(config.time_zone);
            let today_arr = [];
            let earlier_arr = [];
            _.each(noti, (element, index, list) => {
                if(element.profile_picture!=''){
                    element.profile_picture = config.aws.prefix + config.aws.s3.customerBucket + '/' + element.profile_picture
                }
                let created_at = timeZone(new Date(element.created_at)).tz(config.time_zone)
                let duration = moment.duration(todayDate.diff(created_at));
                if(today == timeZone(new Date(element.created_at)).tz(config.time_zone).format('YYYY-MM-DD')){
                    const hours = parseInt(duration.asHours());
                    const minutes = parseInt(duration.asMinutes()) - hours * 60;
                    if(hours > 0){
                        element.created_at = hours + "hrs " + minutes + "mins"
                    }
                    else{
                        element.created_at = minutes + "mins";
                    }
                    today_arr.push(element)
                }
                else{
                    const days = parseInt(duration.asDays());
                    if(days > 0){
                        element.created_at = days > 1 ? days + "days " : days+ "day";
                    }
                    else{
                        const hours = parseInt(duration.asHours());
                        element.created_at = hours > 1 ? hours + "hrs" : hours + "hr";
                    }
                    earlier_arr.push(element)
                }
            });

            resolve({
                today: today_arr,
                earlier: earlier_arr
            })
            return;
        } catch (error) {
            reject(error)
            return
        }
    })
};

/*
    Name : Delete
    Purpose : Delete
    Original Author : Gaurav Patel
    Created At : 29th Sep 2020
*/
const deleteNoti = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            await query.removeMultiple(dbConstants.dbSchema.notification_logs, {user_id: {$in:[requestParam.user_id]}})
            resolve({})
            return;
        } catch (error) {
            reject(error)
            return
        }
    })
};

/*
    Name : mark as read
    Purpose : mark as read
    Original Author : Gaurav Patel
    Created At : 25th May 2021
*/
const allRead = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            await query.updateMultiple(dbConstants.dbSchema.notification_logs, {read_status:'read'}, {
                'user_id': {
                    $in: [requestParam.user_id]
                }
            }); 
            resolve({})
            return;
        } catch (error) {
            reject(error)
            return
        }
    })
};

/*
    Name : mark as read
    Purpose : mark as read
    Original Author : Gaurav Patel
    Created At : 1st Jul 2021
*/
const singleRead = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            await query.updateMultiple(dbConstants.dbSchema.notification_logs, {read_status:'read'}, {
                'notification_log_id': {
                    $in: [requestParam.notification_log_id]
                }
            }); 
            resolve({})
            return;
        } catch (error) {
            reject(error)
            return
        }
    })
};

module.exports = {
    sendNotification,
    setLogs,
    list,
    deleteNoti,
    allRead,
    singleRead
};