'use strict';

const config = require('./../../config');
const logger = require('./../../utils/logger');
const errors = require('./../../utils/dz-errors-api');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator-api');
let async = require('async');
let _ = require('underscore');

/*
    Name : Parking spot type List
    Purpose : Parking spot type List
    Original Author : Gaurav Patel
    Created At : 12th Mar 2021
*/
const allSpotList = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            let types = await query.selectWithAnd(dbConstants.dbSchema.parking_spot_types, { status: 'active'}, { _id: 0, parking_spot_type_id: 1, title: 1 }, { order_no: 1 });
            types = JSON.parse(JSON.stringify(types));
            _.each(types, (element, index, list) => {
                element['title'] = element.title[requestParam.code] || '';
            });

            let coverage = await query.selectWithAnd(dbConstants.dbSchema.parking_spot_coverages, { status: 'active'}, { _id: 0, parking_spot_coverage_id: 1, title: 1 }, { order_no: 1 });
            coverage = JSON.parse(JSON.stringify(coverage));
            _.each(coverage, (element, index, list) => {
                element['title'] = element.title[requestParam.code] || '';
            });

            let terrain = await query.selectWithAnd(dbConstants.dbSchema.parking_spot_terrains, { status: 'active'}, { _id: 0, parking_spot_terrain_id: 1, title: 1 }, { order_no: 1 });
            terrain = JSON.parse(JSON.stringify(terrain));
            _.each(terrain, (element, index, list) => {
                element['title'] = element.title[requestParam.code] || '';
            });

            let amenity = await query.selectWithAnd(dbConstants.dbSchema.parking_spot_amenities, { status: 'active'}, { _id: 0, parking_spot_amenity_id: 1, title: 1, icon:1 }, { order_no: 1 });
            amenity = JSON.parse(JSON.stringify(amenity));
            _.each(amenity, (element, index, list) => {
                element['title'] = element.title[requestParam.code] || '';
                element['icon'] = element.icon != '' ? config.aws.prefix + config.aws.s3.amenityBucket + '/' + element.icon : '';
            });

            resolve({
                spot_type:types,
                spot_coverage:coverage,
                spot_terrain:terrain,
                spot_amenity:amenity,
            })
            return;
        } catch (error) {
            reject(error)
            return
        }
    })
};


module.exports = {
    allSpotList
};