'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.role_id){
		columnAndValue.role_id = req.query.role_id
	}
	query.selectWithAndFilter(dbConstants.dbSchema.access_rights, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, access) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.access_rights);
            done(errors.internalServer(true), null);
            return;
        }
        if(req.query.role_id){
            done(null, access.length > 0 ? access[0] : {});
            return;
        }
        done(null,access)
    });
};

const submit = (requestParam, done) => {
    query.updateSingle(dbConstants.dbSchema.access_rights,{module: requestParam.access_rights}, { 'role_id':requestParam.role_id},function (error, access) {
        if (error) {
            logger('Error: can not update access_rights');
            done(error, null);
            return;
        }
        done(null, access);
    });
};

const check = (requestParam, done) => {
    query.selectWithAndOne(dbConstants.dbSchema.access_rights, requestParam, {
        _id:0,
    }, async (error, response) => {
        if (error) {
            logger('Error: can not get access_rights');
            done(error, null);
            return;
        }
        done(null, response ? response.module : {});
    });
};



module.exports = {
	get,
    submit,
    check
};