'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
let fs = require('fs');
const moment = require('moment');
const config = require("./../../config");

// for DB Backup
var nrc = require('node-run-cmd');
var zipFolder = require('zip-folder');
var download = require('download-file');
var extract = require('extract-zip');
const listAllObjects = require('s3-list-all-objects');

const S3Handler = require('./../../utils/s3-handler');
const s3Handler = new S3Handler();

const restoreBackup = (requestParam, done) => {
    let filename = moment().format('DD-MM-YYYY-HH:mm:ss');
    nrc.run(config.aws.mongoDumpUrl+' '+config.aws.rootDir+filename).then(function(exitCodes) {
        zipFolder(config.aws.rootDir+filename, config.aws.rootDir+filename+'.zip', function(err) {
            if(err) {
                console.log('oh no!', err);
            } else {
                fs.readFile(config.aws.rootDir+filename+'.zip', function read(error, data) {
                    if (error) {
                        console.log(error)
                        logger('Error: while reading file.');
                        done(errors.internalServer(true), null);
                        return;
                    }
                    s3Handler.writeFile(data, filename+'.zip', config.aws.s3.dbBucket, 'db', (error, imagePath) => {
                        if (error) {
                            logger('Error: upload photo from aws s3 bucket. ' + error);
                            done(errors.internalServer(true), null);
                            return;
                        } else {
                            nrc.run('rm -rf '+config.aws.rootDir+'*').then(function(exitCodes) {
                                var options = {
                                    directory: config.aws.rootDir,
                                    filename: moment().format('DD-MM-YYYY-HH:mm:ss')+'.zip'
                                }
                                download(requestParam.url, options, function(err){
                                    if (err){
                                        logger('Error: while download file.');
                                        done(errors.internalServer(true), null);
                                        return;
                                    }
                                    extract(config.aws.rootDir+options.filename, {dir: config.aws.rootDir}, function (err) {
                                        if (err){
                                            logger('Error: while extract file.');
                                            done(errors.internalServer(true), null);
                                            return;
                                        }
                                        nrc.run(config.aws.mongoRestoreUrl+' '+config.aws.rootDir+'parksie').then(function(exitCodes) {
                                            nrc.run('rm -rf '+config.aws.rootDir+'*').then(function(exitCodes) {
                                                listDB(done);
                                            });
                                        }, function(err) {
                                            console.log('Command failed to run with error restore DB: ', err);
                                        });
                                    })
                                })
                            });
                        }
                    })
                })
            }
        });
    }, function(err) {
        console.log('Command failed to run with error create DB: ', err);
    });
};

const deleteBackup = (requestParam, done) => {
    requestParam.obj.filename = requestParam.obj.filename.replace('parksie/database/', '');
    s3Handler.deleteFile(requestParam.obj.filename, config.aws.s3.dbBucket, (error, imagePath) => {
        if (error) {
            logger('Error: while delete from aws s3 bucket. ' + error);
            done(errors.internalServer(true), null);
            return;
        } else {
            listDB(done);
        }
    })
}

const createNewBackup = (done) => {
    let filename = moment().format('DD-MM-YYYY-HH:mm:ss');
    nrc.run(config.aws.mongoDumpUrl+' '+config.aws.rootDir+filename).then(function(exitCodes) {
        zipFolder(config.aws.rootDir+filename, config.aws.rootDir+filename+'.zip', function(err) {
            if(err) {
                console.log('oh no!', err);
            } else {
                fs.readFile(config.aws.rootDir+filename+'.zip', function read(error, data) {
                    if (error) {
                        console.log(error)
                        logger('Error: while reading file.');
                        done(errors.internalServer(true), null);
                        return;
                    }
                    s3Handler.writeFile(data, filename+'.zip', config.aws.s3.dbBucket, 'db', (error, imagePath) => {
                        if (error) {
                            logger('Error: upload photo from aws s3 bucket. ' + error);
                            done(errors.internalServer(true), null);
                            return;
                        } else {
                            nrc.run('rm -rf '+config.aws.rootDir+'*').then(function(exitCodes) {
                                listDB(done);
                            });
                        }
                    })
                })
            }
        });
    }, function(err) {
        console.log('Command failed to run with error: ', err);
    });
};

const listDB = (done) => {
    const units = ['bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];

    function niceBytes(x){
        let l = 0, n = parseInt(x, 10) || 0;
        while(n >= 1024 && ++l)
          n = n/1024;
        return(n.toFixed(n >= 10 || l < 1 ? 0 : 1) + ' ' + units[l]);
    }
    console.log('call list DB');
    // Get with extra s3 options.
    listAllObjects({ bucket: config.aws.s3.assetUpload.bucket, prefix : 'parksie/database/', s3options: {accessKeyId:config.aws.keyId, secretAccessKey:config.aws.key}}, function(err, data) {
        let dbList = _.filter(data, function(item){ return item.Key != 'parksie/database/'; });
        let dbFilesArr = [];
        _.each(dbList, function(item, index) {
            let name = /[^/]*$/.exec(item.Key)[0]
            dbFilesArr.push({ id: index, name:name ,filename: item.Key, url: config.aws.prefix+config.aws.s3.assetUpload.bucket+'/'+item.Key, size: niceBytes(item.Size) });
        });
        done(null, dbFilesArr);
    });
};


module.exports = {
    listDB,
    createNewBackup,
    deleteBackup,
    restoreBackup,
};