'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const timeZone = require('moment-timezone');

const get = function(req,done){
	let columnAndValue={}
	query.selectWithAndFilter(dbConstants.dbSchema.login_logs, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, login_logs) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.login_logs);
            done(errors.internalServer(true), null);
            return;
        }
        login_logs = _.sortBy(login_logs, 'login_date').reverse()
        login_logs = JSON.parse(JSON.stringify(login_logs))
        _.each(login_logs, (elem) => {
            if(!elem.login_date){
                elem.login_date = ''
            }
            else{
                elem.login_date = timeZone(new Date(elem.login_date)).tz(req.headers.time_zone).format('lll');
            }
            if(!elem.logout_date){
                elem.logout_date = ''
            }
            else{
                elem.logout_date = timeZone(new Date(elem.logout_date)).tz(req.headers.time_zone).format('lll');
            }
        })
        done(null,login_logs)
    });
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.removeMultiple(dbConstants.dbSchema.login_logs, {
            'loginlog_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            done(null, data);
        });        
    }
    else
    {
        done(null, {}); 
    }
};


module.exports = {
	get,
	action
};