'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const Parking_spot_terrain = require('./../../models/parking-spot-terrain');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.parking_spot_terrain_id){
		columnAndValue.parking_spot_terrain_id = req.query.parking_spot_terrain_id
	}
	query.selectWithAndFilter(dbConstants.dbSchema.parking_spot_terrains, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, spot_type) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.parking_spot_terrains);
            done(errors.internalServer(true), null);
            return;
        }
        if(req.query.parking_spot_terrain_id){
            spot_type = spot_type[0]
            spot_type = JSON.parse(JSON.stringify(spot_type));
            query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'}, {}, function(error, languages) {
                _.each(languages, (element, index, list) => {
                    spot_type[element.code] = spot_type.title[element.code];
                });
                done(null,spot_type)
            });
        }
        else{
            done(null,spot_type)
        }
    });
};

const create = function(requestParam,done){
	let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
        });
        requestParam.title = title;
        query.insertSingle(dbConstants.dbSchema.parking_spot_terrains,requestParam,function (error, spot_type) {
            if (error) {
                logger('Error: can not create spot_type');
                done(error, null);
                return;
            }
            done(null, spot_type);
        });
    });
};

const update = function(requestParam,done){
    let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
        });
        requestParam.title = title;
    	query.updateSingle(dbConstants.dbSchema.parking_spot_terrains,requestParam, { 'parking_spot_terrain_id':requestParam.parking_spot_terrain_id},function (error, spot_type) {
    		if (error) {
    			logger('Error: can not update spot_type');
    			done(error, null);
    			return;
    		}
    		done(null, spot_type);
    	});
    });
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.removeMultiple(dbConstants.dbSchema.parking_spot_terrains, {
            'parking_spot_terrain_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            done(null, data);
        });        
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.parking_spot_terrains, columnsToUpdate, {
            'parking_spot_terrain_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update spot_type');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};


module.exports = {
	get,
	create,
	action,
	update,
};