'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
const queryApi = require('./../../utils/query-creator-api');
let async = require('async');
let _ = require('underscore');
const config = require('./../../config');
let fs = require('fs');
let moment = require('moment');
const timeZone = require('moment-timezone');
const LD = require('lodash');
const cronHandler = require('./../../model_handlers/api/cron-handler');

const get = function(req,done){
    let columnAndValue = {
        user_type: {$in: ['owner', 'both']}
    }
    if(req.query.owner_id && req.query.owner_id != 'undefined'){
        columnAndValue.customer_id = req.query.owner_id
    }
    if(req.query.payment_frequency_id && req.query.payment_frequency_id != 'undefined'){
        columnAndValue['bank_details.payment_frequency_id'] = req.query.payment_frequency_id
    }
	query.selectWithAndFilter(dbConstants.dbSchema.customers, columnAndValue, {
        _id: 0,
        customer_id:1,
        name:1,
        bank_details:1,
        bank_details_added:1
    }, {created_at:-1}, {}, async (error, response) => {
        if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.payouts);
            done(errors.internalServer(true), null);
            return;
        }
        response = JSON.parse(JSON.stringify(response))
        await Promise.all(response.map(async (elem) => {
            if(elem.bank_details_added && elem.bank_details_added == true){
                let payment_frequency = await queryApi.selectWithAndOne(dbConstants.dbSchema.payment_frequencies, {payment_frequency_id: elem.bank_details.payment_frequency_id}, { _id: 0, payment_frequency_id:1, title:1 }, { created_at: -1 });
                if(payment_frequency){
                    elem.payment_frequency = payment_frequency.title.EN
                }
                else{
                    elem.payment_frequency = ''
                }
            }
            else{
                elem.payment_frequency = ''
                elem.bank_details = {}
            }

            let nextDate = ''
            if(elem.payment_frequency == 'Monthly'){
                var d = new Date();
                d.setMonth(d.getMonth() + 1, 1);
                nextDate = new Date(d)
            }
            if(elem.payment_frequency == 'Annually'){
                var d = new Date();
                let yr = d.getFullYear()
                yr = parseFloat(yr)
                nextDate = new Date(yr+1, 0, 1)
            }
            if(elem.payment_frequency == 'Quarterly'){
                var today = new Date();
                var quarter = Math.floor((today.getMonth() + 3) / 3);
                if (quarter == 4) {
                    nextDate = new Date(today.getFullYear() + 1, 0, 1);
                } else {
                    nextDate = new Date(today.getFullYear(), quarter * 3, 1);
                }
            }
            if(nextDate!=''){
                elem.next_payment_date = timeZone(new Date(nextDate)).tz(req.headers.time_zone).format('ll')
            }
            else{
                elem.next_payment_date = ''
            }

            let reservations = await queryApi.selectWithAnd(dbConstants.dbSchema.reservations, { status: 'completed', owner_id: elem.customer_id, is_owner_paid:false}, { _id: 0, owner_id: 1, reservation_id: 1, owner_earn:1 }, { created_at: -1 });
            elem.next_payout_total = parseFloat(parseFloat(LD.sumBy(reservations, 'owner_earn')).toFixed(2))

            let last = await queryApi.selectWithAndOne(dbConstants.dbSchema.payouts, { status: 'success', owner_id: elem.customer_id}, { _id: 0, owner_id: 1, last_payment_date: 1, last_payout_total:1, next_payment_date:1 }, { created_at: -1 });
            if(last){
                elem.last_payment_date = timeZone(new Date(last.last_payment_date)).tz(req.headers.time_zone).format('ll')
                elem.last_payout_total = last.last_payout_total
            }
            else{
                elem.last_payment_date = ''
                elem.last_payout_total = 0
            }
        }))
        done(null, response)
    });
};

const getHistory = function(req,done){
    let columnAndValue = {}
    if(req.query.owner_id && req.query.owner_id != 'undefined'){
        columnAndValue.owner_id = req.query.owner_id
    }
    if(req.query.status && req.query.status != 'undefined'){
        columnAndValue.status = req.query.status
    }
    if(req.query.payment_frequency_id && req.query.payment_frequency_id != 'undefined'){
        columnAndValue['bank_details.payment_frequency_id'] = req.query.payment_frequency_id
    }
    query.selectWithAndFilter(dbConstants.dbSchema.payouts, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, response) => {
        if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.payouts);
            done(errors.internalServer(true), null);
            return;
        }
        response = JSON.parse(JSON.stringify(response))
        _.each(response, (elem) => {
            elem.last_payment_date = timeZone(new Date(elem.last_payment_date)).tz(req.headers.time_zone).format('ll');
            elem.next_payment_date = timeZone(new Date(elem.next_payment_date)).tz(req.headers.time_zone).format('ll');
        });
        done(null, response)
    });
};

const manually = function(requestParam,done){
    query.selectWithAndOne(dbConstants.dbSchema.customers, {customer_id: requestParam.owner_id}, {
        _id: 0,
        bank_details:1
    }, (error, response) => {
        if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.payouts);
            done(errors.internalServer(true), null);
            return;
        }
        query.selectWithAndOne(dbConstants.dbSchema.payment_frequencies, {payment_frequency_id: response.bank_details.payment_frequency_id}, {
            _id: 0,
            payment_frequency_id:1,
            title:1
        }, async (error, payment_fre) => {
            if (error) {
                logger('Error: can not get ', dbConstants.dbSchema.payouts);
                done(errors.internalServer(true), null);
                return;
            }
            if(response && payment_fre){
                let type = (payment_fre.title.EN.trim()).toLowerCase();
                let payout = await cronHandler.payout({type: type, owner_id: requestParam.owner_id})
            }
            done(null, {})
        });
    });
};


module.exports = {
    get,
	getHistory,
    manually,
};