'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const Tax = require('./../../models/tax');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.tax_id){
		columnAndValue.tax_id = req.query.tax_id
	}
	query.selectWithAndFilter(dbConstants.dbSchema.taxes, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, tax) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.taxes);
            done(errors.internalServer(true), null);
            return;
        }
        if(req.query.tax_id){
            tax = tax[0]
            tax = JSON.parse(JSON.stringify(tax));
            query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'}, {}, function(error, languages) {
                _.each(languages, (element, index, list) => {
                    tax[element.code] = tax.title[element.code];
                });
                done(null,tax)
            });
        }
        else{
            done(null,tax)
        }
    });
};

const create = function(requestParam,done){
	let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
        });
        requestParam.title = title;
        query.insertSingle(dbConstants.dbSchema.taxes,requestParam,function (error, tax) {
            if (error) {
                logger('Error: can not create tax');
                done(error, null);
                return;
            }
            done(null, tax);
        });
    });
};

const update = function(requestParam,done){
    let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
        });
        requestParam.title = title;
    	query.updateSingle(dbConstants.dbSchema.taxes,requestParam, { 'tax_id':requestParam.tax_id},function (error, tax) {
    		if (error) {
    			logger('Error: can not update tax');
    			done(error, null);
    			return;
    		}
    		done(null, tax);
    	});
    });
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.removeMultiple(dbConstants.dbSchema.taxes, {
            'tax_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            done(null, data);
        });        
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.taxes, columnsToUpdate, {
            'tax_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update tax');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};


module.exports = {
	get,
	create,
	action,
	update,
};