'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const config = require('./../../config');
const Vehicle_type = require('./../../models/vehicle-type');
const commonHandler = require('./../../model_handlers/backend/common-handler');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.vehicle_type_id){
		columnAndValue.vehicle_type_id = req.query.vehicle_type_id
	}
	query.selectWithAndFilter(dbConstants.dbSchema.vehicle_types, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, vehicle) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.vehicle_types);
            done(errors.internalServer(true), null);
            return;
        }
        if(req.query.vehicle_type_id){
            vehicle = vehicle[0]
            vehicle = JSON.parse(JSON.stringify(vehicle));
            vehicle.icon = vehicle.icon != '' ? config.aws.prefix + config.aws.s3.vehicleTypeBucket + '/' + vehicle.icon : '';
            query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'}, {}, function(error, languages) {
                _.each(languages, (element, index, list) => {
                    vehicle[element.code] = vehicle.title[element.code];
                    vehicle['desc_'+element.code] = vehicle.description[element.code];
                });
                done(null,vehicle)
            });
        }
        else{
            done(null,vehicle)
        }
    });
};

const create = async function(requestParam, req, done){
	let title = new Object();
    let description = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, async function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
            description[element.code] = requestParam['desc_'+element.code];
        });
        requestParam.title = title;
        requestParam.description = description;
        requestParam.icon = await new Promise((resolve, reject) => {
            commonHandler.uploadImage(req.files.icon, config.aws.s3.vehicleTypeBucket, (error, path) => {
                resolve(path)
            });
        });
        query.insertSingle(dbConstants.dbSchema.vehicle_types,requestParam,function (error, tax) {
            if (error) {
                logger('Error: can not create tax');
                done(error, null);
                return;
            }
            done(null, tax);
        });
    });
};

const update = function(requestParam, req, done){
    let title = new Object();
    let description = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, async function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
            description[element.code] = requestParam['desc_'+element.code];
        });
        requestParam.title = title;
        requestParam.description = description;
        if (requestParam.change_logo) {
            query.selectWithAndOne(dbConstants.dbSchema.vehicle_types, {
                vehicle_type_id: requestParam.vehicle_type_id
            }, {
                vehicle_type_id: 1,
                icon: 1,
            }, async(error, user) => {
                let fileObjects = [];
                if(requestParam.change_logo){
                    fileObjects.push({
                        Key: 'parksie/vehicle-types/' + /[^/]*$/.exec(user.icon)[0]
                    })
                }
                commonHandler.removeMultipleImages(fileObjects, async(error, path) => {
                    requestParam.icon = await new Promise((resolve, reject) => {
                        commonHandler.uploadImage(req.files.icon, config.aws.s3.vehicleTypeBucket, (error, path) => {
                            resolve(path)
                        });
                    });
                    query.updateSingle(dbConstants.dbSchema.vehicle_types,requestParam, { 'vehicle_type_id':requestParam.vehicle_type_id},function (error, tax) {
                        if (error) {
                            logger('Error: can not update tax');
                            done(error, null);
                            return;
                        }
                        done(null, tax);
                    });
                });
            });
        }
        else{
            delete requestParam.icon
        	query.updateSingle(dbConstants.dbSchema.vehicle_types,requestParam, { 'vehicle_type_id':requestParam.vehicle_type_id},function (error, tax) {
        		if (error) {
        			logger('Error: can not update tax');
        			done(error, null);
        			return;
        		}
        		done(null, tax);
        	});
        }
    });
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.selectWithAnd(dbConstants.dbSchema.vehicle_types, {
            vehicle_type_id: {
                $in: requestParam.ids
            }
        }, {
            _id: 0,
            vehicle_type_id: 1,
            icon: 1,
        }, (error, user) => {
            let fileObjects = [];
            for (var i = 0; i < user.length; i++) {
                fileObjects.push({
                    Key: 'parksie/vehicle-types/' + /[^/]*$/.exec(user[i].icon)[0]
                })
            }
            commonHandler.removeMultipleImages(fileObjects, (error, path) => {
                query.removeMultiple(dbConstants.dbSchema.vehicle_types, {
                    'vehicle_type_id': {
                        $in: requestParam['ids']
                    }
                }, function(error, data) {
                    if (error) {
                        logger('Error: can not delete ');
                        done(error, null);
                        return;
                    }
                    done(null, data);
                });
            });
        });         
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.vehicle_types, columnsToUpdate, {
            'vehicle_type_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update tax');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};


module.exports = {
	get,
	create,
	action,
	update,
};