var mongoose = require('mongoose');
var Schema = mongoose.Schema;
const idGenerator = require('./../utils/id-generator');

var customerSchema = new Schema({
    customer_id: {
        type: String,
        default: ''
    },
    user_type: {
        type: String,
        enum: ['seeker', 'owner', 'both', ''],
        default:''
    },
    language_id: {
        type: String,
        default: ''
    },
    measurement_id: {
        type: String,
        default: ''
    },
    social_id: {
        type: String,
        default: ''
    },
    register_type: {
        type: String,
        enum: ['mobile', 'google', 'apple'],
        default: 'mobile'
    },
    name: {
        type: String,
        default: ''
    },
    surname: {
        type: String,
        default: ''
    },
    email: {
        type: String,
        default: ''
    },
    password: {
        type: String,
        default: ''
    },
    mobile_country_code: {
        type: String,
        default: ''
    },
    mobile: {
        type: String,
        default: ''
    },
    profile_picture: {
        type: String,
        default: ''
    },
    player_id: {
        type: String,
        default: ''
    },
    device_token: {
        type: String,
        default: ''
    },
    device_type: {
        type: String,
        enum:["android", "ios", "web"],
        default: 'web'
    },
    device_id: {
        type: String,
        default: ''
    },
    otp: {
        type: Number,
        default: 0
    },
    stripe_profile_id: {
        type: String,
        default: ''
    },
    stripe_account_id: {
        type: String,
        default: ''
    },
    cards: {
        type: Array,
        default: []
    },
    default_card: {
        type: String,
        default: ''
    },
    reservation_updates: {
        type: Object,
        default: {
            sms: true,
            in_app:true
        }
    },
    payment_updates: {
        type: Object,
        default: {
            sms: true,
            in_app:true
        }
    },
    message_updates: {
        type: Object,
        default: {
            sms: true,
            in_app:true
        }
    },
    general_updates: {
        type: Object,
        default: {
            sms: true,
            in_app:true
        }
    },
    invitation_code: {
        type: String,
        default: function() {
            var result           = '';
            var characters       = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
            var charactersLength = characters.length;
            for ( var i = 0; i < 6; i++ ) {
                result += characters.charAt(Math.floor(Math.random() * charactersLength));
            }
           return result;
        }
    },
    bank_details:{
        type: Object,
        default:{}
    },
    bank_details_added:{
        type: Boolean,
        default:false
    },
    stripe_connected_account_id: {
        type: String,
        default: '',
    },
    stripe_customer_account_id: {
        type: String,
        default: '',
    },
    stripe_connected_account_link_url: {
        type: String,
        default: '',
    },
    status: {
        type: String,
        enum: ['active', 'inactive', 'flag'],
        default:'active'
    },
    last_login: {
        type: Date
    },
    created_at: {
        type: Date,
        default: Date.now
    },
    updated_at: {
        type: Date,
        default: Date.now
    }
});



// // Execute before each user.save() call
customerSchema.pre('save', function(callback) {
    idGenerator.generateId('customers', 'customer_id', 'CUS', (err, ID) => {
        this.customer_id = ID;
        callback();
    });
});

var Customer = mongoose.model('Customer', customerSchema);
module.exports = Customer;