// grab the things we need
var mongoose = require('mongoose');
var Schema = mongoose.Schema;
const idGenerator = require('./../utils/id-generator');

// create a schema
var reservationSchema = new Schema({
    reservation_id: {
        type: String,
        default:''
    },
    transaction_id: {
        type: String,
        default:''
    },
    extend_transaction_id: {
        type: String,
        default:''
    },
    over_charge_transaction_id: {
        type: String,
        default:''
    },
    customer_id: {
        type: String,
        default:''
    },
    owner_id: {
        type: String,
        default:''
    },
    property_id: {
        type: String,
        default:''
    },
    vehicle_id: {
        type: String,
        default:''
    },
    card_id: {
        type: String,
        default:''
    },
    price: {
        type: Number,
        default:0
    },
    overtime_charge: {
        type: Number,
        default:0
    },
    company_percentage: {
        type: Number,
        default:0
    },
    company_earn: {
        type: Number,
        default:0
    },
    owner_earn: {
        type: Number,
        default:0
    },
    tax_info:{
        type: Object,
        default:{}
    },
    driver_details: {
        type: Object,
        default:{}
    },
    from_date: {
        type: Date,
        default:Date.now
    },
    to_date: {
        type: Date,
        default:Date.now
    },
    parked_date: {
        type: Date,
        default:Date.now
    },
    is_owner_paid: {
        type: Boolean,
        default: false
    },
    completed_date: {
        type: Date,
        default:Date.now
    },
    book_type: {
        type: String,
        enum: ['parking_meter', 'normal'],
        default:'normal'
    },
    status: {
        type: String,
        enum: ['reserved', 'parked', 'completed', "cancelled"],
        default:'reserved'
    },
    is_refund: {
        type: Boolean,
        enum: false
    },
    created_at: {
        type: Date,
        default: Date.now
    },
    updated_at: {
        type: Date,
        default: Date.now
    }
});


// // Execute before each user.save() call
reservationSchema.pre('save', function(callback) {
    idGenerator.generateId('reservations', 'reservation_id', 'RES', (err, ID) => {
        this.reservation_id = ID;
        callback();
    });
});

var Reservation = mongoose.model('Reservation', reservationSchema);
module.exports = Reservation;