# Stream to Buffer

Concatenate a readable stream's data into a single buffer.

It will concatenate a raw stream to a `Buffer`, an ascii or utf8
encoded stream to a `String`, and an object mode stream to an
`Array`.

## Installing

    npm install --save stream2buffer

## API

```js
var stream2Buffer = require('stream2buffer');
var fs = require('fs');
var Readable = require('stream').Readable;
var assert = require('assert');

// Raw stream
var rawStream = fs.createReadStream('hamlet.txt');
stream2Buffer(rawStream, function (err, buffer) {
  assert.ok(buffer instanceof Buffer);
  assert.ok(Buffer.isBuffer(buffer));
});

// utf8 encoded stream
var encodedStream = fs.createReadStream('hamlet.txt', {
  encoding: 'utf8' // would be the same with ascii here
});
stream2Buffer(encodedStream, function (err, buffer) {
  assert.ok(typeof buffer === 'string');
});

// object mode stream
var objectStream = new Readable({ objectMode: true });
var n = 10;
objectStream._read = function _read () {
  objectStream.push({ "hello": "world" });
  if (!n--)
    objectStream.push(null);
};
stream2Buffer(objectStream, function (err, buffer) {
  assert.ok(buffer instanceof Array);
  assert.ok(Array.isArray(buffer));
});
```

## A fork of

[Jonathan Ong](https://github.com/jonathanong)'s [`stream-to-buffer`](https://github.com/jonathanong/stream-to-buffer/tree/10874042b9c5c1c447889c028fa078a40eed830d).
I forked it because we disagree on how much [`stream-to-buffer` must
know](https://github.com/jonathanong/stream-to-buffer/pull/2)
to work properly.

## License

The MIT License (MIT)

Copyright (c) 2013 Jonathan Ong me@jongleberry.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
