'use strict';

const config = require('./../../config');
const responseCodes = require('./../../helpers/response-codes');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors-api');
const express = require('express');
const router = express.Router();
const authHandler = require('./../../model_handlers/api/auth-handler');
const _ = require('underscore');

/*
    Name : Check social id
    Purpose : Check social id
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.get('/check-social-id', async(req, res) => {
    try {
        if (!req.query.code || !req.query.social_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }
        let response = await authHandler.checkSocialIdExist(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Sign in
    Purpose : Sign in
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/sign-in', async(req, res) => {
    try {
        if (req.body.code && req.body.mobile_country_code && req.body.mobile && req.body.password) {
            let response = await authHandler.signIn(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
	Name : Sign up
	Purpose : Sign up
	Original Author : Gaurav Patel
	Created At : Created At : 24th feb 2021
*/
router.post('/sign-up', async(req, res) => {
    try {
        if(!req.body.password){
            req.body.password = ''
        }
        if(req.body.register_type == 'mobile'){
            if (req.body.code && req.body.name && req.body.mobile_country_code && req.body.mobile && req.body.register_type == 'mobile') {
                let response = await authHandler.signUp(req.body, req);
                jsonResponse(res, responseCodes.OK, errors.noError(), response);
            } else {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        else{
            if (req.body.code && req.body.name && ((req.body.register_type == 'google' || req.body.register_type == 'apple') && req.body.social_id)) {
                let response = await authHandler.signUp(req.body, req);
                jsonResponse(res, responseCodes.OK, errors.noError(), response);
            } else {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});


/*
    Name : Check Email Mobile
    Purpose : Check Email Mobile
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/check-email-mobile-exists', async(req, res) => {
    try {
        if (req.body.code && req.body.mobile_country_code && req.body.mobile && req.body.name) {
            let response = await authHandler.checkEmailMobileExists(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});


/*
    Name : Verify Mobile
    Purpose : Verify Mobile
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/verify-mobile', async(req, res) => {
    try {
        if (req.body.code && req.body.verify_mobile_id && req.body.otp) {
            let response = await authHandler.verifyMobile(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Resend OTP
    Purpose : Resend OTP
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/resend-otp-verify-mobile', async(req, res) => {
    try {
        if (req.body.code && req.body.verify_mobile_id && req.body.name) {
            let response = await authHandler.resendOtpUsingMobileVerify(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Forgot Pwd
    Purpose : Forgot Pwd
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/forgot-password', async(req, res) => {
    try {
        if (req.body.code && req.body.mobile && req.body.mobile_country_code) {
            let response = await authHandler.forgotPassword(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Verify OTP
    Purpose : Verify OTP
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/verify-otp', async(req, res) => {
    try {
        if (req.body.code && req.body.mobile && req.body.mobile_country_code && req.body.otp) {
            let response = await authHandler.verifyOTP(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Reset Pwd
    Purpose : Reset Pwd
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.post('/reset-password', async(req, res) => {
    try {
        if (req.body.code && req.body.mobile && req.body.mobile_country_code && req.body.password) {
            let response = await authHandler.resetPassword(req.body);
            jsonResponse(res, responseCodes.OK, errors.noError(), response);
        } else {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Logout Delete Account
    Purpose : Logout Delete Account
    Original Author : Gaurav Patel
    Created At : 24th feb 2021
*/
router.get('/logout-delete-account', async(req, res) => {
    try {
        if (!req.query.code || !req.query.customer_id || !req.query.action) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }
        let response = await authHandler.logoutDeleteAccount(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Get Profile
    Purpose : Get Profile
    Original Author : Gaurav Patel
    Created At : 5th Mar 2021
*/
router.get('/get-profile', async(req, res) => {
    try {
        if (!req.query.code || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }
        let response = await authHandler.profile({customer_id:req.query.customer_id}, req.query.code);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        console.log(error)
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Update Profile
    Purpose : Update Profile
    Original Author : Gaurav Patel
    Created At : 5th Mar 2021
*/
router.post('/update-profile', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        let response = await authHandler.updateProfile(req.body, req);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        console.log(error)
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Change Password
    Purpose : Change Password
    Original Author : Gaurav Patel
    Created At : 8th Mar 2021
*/
router.post('/change-password', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.old_password || !req.body.new_password)  {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        let response = await authHandler.changePassword(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        console.log(error)
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Update notification update
    Purpose : Update notification update
    Original Author : Gaurav Patel
    Created At : 8th Mar 2021
*/
router.post('/update-notification', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.type || !req.body.action || !req.body.value)  {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        let response = await authHandler.updateNotification(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        console.log(error)
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

module.exports = router;