'use strict';

let _ = require('underscore');
const express = require('express');
const config = require('./../../config');
const responseCodes = require('./../../helpers/response-codes');
const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors-api');
const router = express.Router();
const ownerHandler = require('./../../model_handlers/api/owner-handler');


/*
    Name : reservation List
    Purpose : reservation List
    Original Author : Gaurav Patel
    Created At : 06th APr 2021
*/
router.get('/reservation-list', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id || !req.query.type) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.reservationList(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : List
    Purpose : List
    Original Author : Gaurav Patel
    Created At : 06th APr 2021
*/
router.get('/bank-account-type-payment-list', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.bankAccountTypePaymentList(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Bnak Details
    Purpose : Bnak Details
    Original Author : Gaurav Patel
    Created At : 06th APr 2021
*/
router.post('/add-bank-details', async (req, res) => {
    try {
        req.body.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.body.time_zone = req.headers.time_zone
        }
        // if (!req.body.code || !req.body.customer_id || !req.body.bank_account_holder_name || !req.body.bank_id || !req.body.bank_account_number || !req.body.bank_routing_number || !req.body.bank_account_type_id || !req.body.payment_frequency_id || !req.body.plaid_public_token || !req.body.plaid_account_id) {
        //     jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
        //     return;
        // }
        if (!req.body.code || !req.body.customer_id || !req.body.plaid_public_token || !req.body.plaid_account_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await ownerHandler.bankDetails(req.body);
        //console added by Mukesh Singh
        console.log("add_bank_details_response --> ", response)
        console.log("--------------------------------------------------------------------------")

        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

router.post('/create-stripe-account', async (req, res) => {
    try {
        req.body.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.body.time_zone = req.headers.time_zone
        }
        if (!req.body.code || !req.body.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await ownerHandler.createStripeAcc(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

router.post('/connect-stripe-plaid-acc', async (req, res) => {
    try {
        req.body.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.body.time_zone = req.headers.time_zone
        }
        // if (!req.body.code || !req.body.customer_id || !req.body.bank_account_holder_name || !req.body.bank_id || !req.body.bank_account_number || !req.body.bank_routing_number || !req.body.bank_account_type_id || !req.body.payment_frequency_id || !req.body.plaid_public_token || !req.body.plaid_account_id) {
        //     jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
        //     return;
        // }
        if (!req.body.code || !req.body.customer_id || !req.body.plaid_public_token || !req.body.plaid_account_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await ownerHandler.connectStripePlaidAcc(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Get property list status
    Purpose : Get property list status
    Original Author : Gaurav Patel
    Created At : 13th APr 2021
*/
router.get('/property-status', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.property_id || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.propertyStatus(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Get earning
    Purpose : Get earning
    Original Author : Gaurav Patel
    Created At : 13th APr 2021
*/
router.get('/earning-graph', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id || !req.query.type) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }
        if (req.query.type == 'custom') {
            if (!req.query.start_date || !req.query.end_date) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
                return;
            }
        }

        let response = await ownerHandler.earningGraph(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : delete property
    Purpose : delete property
    Original Author : Gaurav Patel
    Created At : 19th APr 2021
*/
router.get('/delete-property', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id || !req.query.property_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.deleteProperty(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : update property
    Purpose : update property
    Original Author : Gaurav Patel
    Created At : 19th APr 2021
*/
router.get('/update-discoverable-property', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id || !req.query.property_id || !req.query.discoverable_status) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.updateDiscoverableProperty(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : list property
    Purpose : list property
    Original Author : Gaurav Patel
    Created At : 19th APr 2021
*/
router.get('/property-list', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.propertyList(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : rating list
    Purpose : rating list
    Original Author : Gaurav Patel
    Created At : 3rd May 2021
*/
router.get('/rating-list', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.ratingList(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : seeker-rating
    Purpose : seeker-rating
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
router.post('/seeker-rating', async (req, res) => {
    try {
        req.body.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.body.time_zone = req.headers.time_zone
        }
        if (!req.body.code || !req.body.customer_id || !req.body.seeker_id || !req.body.rating || !req.body.rating_status) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await ownerHandler.seekerRating(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        console.log(error)
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : parking spot list property
    Purpose : parking spot list property
    Original Author : Gaurav Patel
    Created At : 19th May 2021
*/
router.get('/parking-spot-list', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id || !req.query.address) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.parkingSpotList(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name :  list property
    Purpose : list property
    Original Author : Gaurav Patel
    Created At : 14th Jun 2021
*/
router.get('/property-list-total-spot', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.propertyListWithTotalSpot(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name :  delete property
    Purpose : delete property
    Original Author : Gaurav Patel
    Created At : 14th Jun 2021
*/
router.get('/delete-property-using-address', async (req, res) => {
    try {
        req.query.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.query.time_zone = req.headers.time_zone
        }
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id || !req.query.address) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await ownerHandler.deletePropertyWithAddress(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name :  Create Plaid Token
    Purpose : Create Plaid Token
    Original Author : Gaurav Patel
    Created At : 02nd Sep 2021
*/
router.post('/plaid-token', async (req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.client_name) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        let response = await ownerHandler.plaidToken(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name :  Cron for overage feed
    Purpose : Cron for overage feed
    Original Author : Gaurav Patel
    Created At : 29nd Sep 2021
*/
router.post('/cron-for-overage-fees', async (req, res) => {
    try {
        req.body.time_zone = config.time_zone
        if (req.headers.time_zone) {
            req.body.time_zone = req.headers.time_zone
        }
        req.body.code = 'EN'
        let response = await ownerHandler.cronForOverageFeed(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

router.post('/strip-acccount-link', async (req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }
        let response = await ownerHandler.stripAccountLink(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

module.exports = router;