'use strict';

const config = require('./../../config');
const responseCodes = require('./../../helpers/response-codes');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors-api');
const labels = require('./../../utils/labels.json');
const express = require('express');
const router = express.Router();
const vehicleHandler = require('./../../model_handlers/api/vehicle-handler');
const _ = require('underscore');


/*
	Name : List
	Purpose : List
	Original Author : Gaurav Patel
	Created At : 8th March 2021
*/
router.get('/list', async(req, res) => {
    try {
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await vehicleHandler.list(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Add
    Purpose : Add
    Original Author : Gaurav Patel
    Created At : 8th March 2021
*/
router.post('/add', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.type || !req.body.vehicle_number || !req.body.vehicle_color_id || !req.body.vehicle_type_id || !req.body.is_default) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        if(req.body.type == 'edit') {
            if(!req.body.vehicle_id){
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }

        let response = await vehicleHandler.add(req.body, req);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Set Default
    Purpose : Set Default
    Original Author : Gaurav Patel
    Created At : 12th March 2021
*/
router.post('/set-default', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.vehicle_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await vehicleHandler.setDefault(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : Delete Vehicle
    Purpose : Delete Vehicle
    Original Author : Gaurav Patel
    Created At : 26th March 2021
*/
router.post('/delete', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.vehicle_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await vehicleHandler.deleteVehicle(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

module.exports = router;