'use strict';

const responseCodes = require('./../../helpers/response-codes');
const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const express = require('express');
const router = express.Router();
const _ = require('underscore');
const accessHandler = require('./../../model_handlers/backend/access-right-handler');

router.get('/get', function(req, res) {
    accessHandler.get(req, function(error, responses) {
        if (error) {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        }
        jsonResponse(res, responseCodes.OK, errors.noError(), responses);
    });
});

router.get('/check', function(req, res) {
    accessHandler.check(req.query, function(error, responses) {
        if (error) {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        }
        jsonResponse(res, responseCodes.OK, errors.noError(), responses);
    });
});

router.post('/submit', function(req, res) {
    accessHandler.submit(req.body, function(error, responses) {
        if (error) {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        }
        jsonResponse(res, responseCodes.OK, errors.noError(), responses);
    });
});


module.exports = router;