'use strict';

const responseCodes =  require('./../../helpers/response-codes');
const logger = require('./../../utils/logger');
const config = require('./../../config');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const express = require('express');
const router = express.Router();
const _ = require('underscore');
const customerHandler = require('./../../model_handlers/backend/customer-handler');

router.post('/create', function (req, res) {
	let requestParam = JSON.parse(req.body.fields);
	customerHandler.create(requestParam, req, function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});


router.get('/get',function (req, res) {
	customerHandler.get(req,function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});


router.post('/action', function (req, res) {
  customerHandler.action(req.body,function (error, response) {
    if (error) {
      jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
      return;
    }
    jsonResponse(res, responseCodes.OK, errors.noError(), response);
  });
});


router.post('/update', function (req, res) {
	let requestParam = JSON.parse(req.body.fields);
	customerHandler.update(requestParam, req, function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});

router.get('/get-bank-details', function (req, res) {
	customerHandler.getBankDetails(req.query, function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});

router.get('/get-property-list', function (req, res) {
	customerHandler.getPropertyList(req.query, function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});

router.get('/get-earning', function (req, res) {
	customerHandler.getEarning(req.query, function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});

router.get('/ratings', function (req, res) {
	req.query.time_zone = config.time_zone
    if(req.headers.time_zone){
        req.query.time_zone = req.headers.time_zone
    }
	customerHandler.ratings(req.query, function (error, response) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), response);
	});
});

module.exports = router;